@@ -1,4 +1,4 @@
-from typing import List, Tuple
+from typing import Tuple
 
 import numpy as np
 from dm_control import composer
@@ -7,6 +7,7 @@ from dm_control import mjcf
 from dexterity.hints import MjcfElement
 from dexterity.models.hands import dexterous_hand
 from dexterity.models.hands import shadow_hand_e_constants as consts
+from dexterity.utils import mjcf_utils
 from dexterity.utils import mujoco_utils
 
 _PALM_UPRIGHT_POS = (0.0, 0.2, 0.1)
@@ -58,7 +59,7 @@ class ShadowHandSeriesE(dexterous_hand.DexterousHand):
         return self._mjcf_root.model
 
     @composer.cached_property
-    def root_body(self):
+    def root_body(self) -> MjcfElement:
         return self._mjcf_root.find("body", "forearm")
 
     @composer.cached_property
@@ -66,27 +67,27 @@ class ShadowHandSeriesE(dexterous_hand.DexterousHand):
         return tuple(self.mjcf_model.find_all("body"))
 
     @property
-    def joints(self) -> List[MjcfElement]:
+    def joints(self) -> Tuple[MjcfElement, ...]:
         return self._joints
 
     @property
-    def actuators(self) -> List[MjcfElement]:
+    def actuators(self) -> Tuple[MjcfElement, ...]:
         return self._actuators
 
     @property
-    def tendons(self) -> List[MjcfElement]:
+    def tendons(self) -> Tuple[MjcfElement, ...]:
         return self._tendons
 
     @property
-    def joint_torque_sensors(self) -> List[MjcfElement]:
+    def joint_torque_sensors(self) -> Tuple[MjcfElement, ...]:
         return self._joint_torque_sensors
 
     @property
-    def fingertip_sites(self) -> List[MjcfElement]:
+    def fingertip_sites(self) -> Tuple[MjcfElement, ...]:
         return self._fingertip_sites
 
     @property
-    def joint_groups(self) -> List[dexterous_hand.JointGrouping]:
+    def joint_groups(self) -> Tuple[dexterous_hand.JointGrouping, ...]:
         return self._joint_groups
 
     # ================= #
@@ -117,20 +118,14 @@ class ShadowHandSeriesE(dexterous_hand.DexterousHand):
             )
         return consts.POSITION_TO_CONTROL @ qpos
 
-    def set_joint_angles(self, physics: mjcf.Physics, joint_angles: np.ndarray) -> None:
-        """Sets the joints of the hand to a given configuration."""
-        physics.bind(self._joints).qpos = joint_angles
-
-    def sample_joint_angles(
-        self, physics: mjcf.Physics, random_state: np.random.RandomState
-    ) -> np.ndarray:
-        qpos = random_state.uniform(*physics.bind(self._joints).range.T)
+    def set_joint_angles(self, physics: mjcf.Physics, qpos: np.ndarray) -> None:
+        physics.bind(self._joints).qpos = qpos
 
+    def _postprocess_sampled_joint_angles(self, qpos: np.ndarray) -> np.ndarray:
         # Ensure coupled joints have the same joint values.
         for coupled_ids in consts.COUPLED_JOINT_IDS:
             val = qpos[coupled_ids[-1]]
             qpos[coupled_ids] = val
-
         return qpos
 
     # ================= #
@@ -139,34 +134,23 @@ class ShadowHandSeriesE(dexterous_hand.DexterousHand):
 
     def _parse_mjcf_elements(self) -> None:
         """Parses MJCF elements that will be exposed as attributes."""
-        # Parse joints.
-        self._joints = self._mjcf_root.find_all("joint")
-        if not self._joints:
-            raise ValueError("No joints found in the MJCF model.")
-
-        # Parse actuators.
-        self._actuators = self._mjcf_root.find_all("actuator")
-        if not self._actuators:
-            raise ValueError("No actuators found in the MJCF model.")
-
-        # Parse tendons.
-        self._tendons = self._mjcf_root.find_all("tendon")
-        if not self._tendons:
-            raise ValueError("No tendons found in the MJCF model.")
+        self._joints = mjcf_utils.safe_find_all(self.mjcf_model, "joint")
+        self._actuators = mjcf_utils.safe_find_all(self.mjcf_model, "actuator")
+        self._tendons = mjcf_utils.safe_find_all(self.mjcf_model, "tendon")
 
         # Create joint groups.
-        self._joint_groups = []
+        joint_groups = []
         for name, group in consts.JOINT_GROUP.items():
             joint_group = dexterous_hand.JointGrouping(
                 name=name,
                 joints=tuple([joint for joint in self._joints if joint.name in group]),
             )
-            self._joint_groups.append(joint_group)
+            joint_groups.append(joint_group)
+        self._joint_groups = tuple(joint_groups)
 
     def _add_fingertip_sites(self) -> None:
         """Adds sites to the tips of the fingers of the hand."""
-        self._fingertip_sites: List[mjcf.Element] = []
-
+        fingertip_sites = []
         for tip_name in consts.FINGERTIP_NAMES:
             tip_elem = self._mjcf_root.find("body", tip_name)
             if tip_elem is None:
@@ -181,7 +165,8 @@ class ShadowHandSeriesE(dexterous_hand.DexterousHand):
                 rgba="1 0 0 1",
                 group=composer.SENSOR_SITES_GROUP,
             )
-            self._fingertip_sites.append(tip_site)
+            fingertip_sites.append(tip_site)
+        self._fingertip_sites = tuple(fingertip_sites)
 
     def _add_sensors(self) -> None:
         """Add sensors to the hand's MJCF model."""
@@ -190,8 +175,7 @@ class ShadowHandSeriesE(dexterous_hand.DexterousHand):
 
     def _add_torque_sensors(self) -> None:
         """Adds torque sensors to the joints of the hand."""
-        self._joint_torque_sensors = []
-
+        joint_torque_sensors = []
         for joint_elem in self._joints:
             site_elem = joint_elem.parent.add(
                 "site",
@@ -208,4 +192,5 @@ class ShadowHandSeriesE(dexterous_hand.DexterousHand):
                 site=site_elem,
                 name=joint_elem.name + "_torque",
             )
-            self._joint_torque_sensors.append(torque_sensor_elem)
+            joint_torque_sensors.append(torque_sensor_elem)
+        self._joint_torque_sensors = tuple(joint_torque_sensors)
