@@ -69,7 +69,7 @@ class Task(composer.Task):
         del random_state  # Unused.
 
         for eff in self._hand_effectors:
-            e_cmd = action[self._find_effector_indices(eff, physics)]
+            e_cmd = action[np.asarray(self._find_effector_indices(eff, physics))]
             eff.set_control(physics, e_cmd)
 
     def action_spec(
@@ -184,7 +184,7 @@ class GoalTask(Task):
                 if physics.data.time - self._solve_start_time > self._max_time_per_goal:
                     self._exceeded_single_goal_time = True
 
-    def should_terminate_episode(self, physics):
+    def should_terminate_episode(self, physics: mjcf.Physics) -> bool:
         del physics  # Unused.
         if self._successes >= self._successes_needed or (
             self._max_time_per_goal is not None and self._exceeded_single_goal_time
